package balance

import (
	"git.frostfs.info/TrueCloudLab/frostfs-node/internal/logs"
	frostfsContract "git.frostfs.info/TrueCloudLab/frostfs-node/pkg/morph/client/frostfs"
	balanceEvent "git.frostfs.info/TrueCloudLab/frostfs-node/pkg/morph/event/balance"
	"go.uber.org/zap"
)

// Process lock event by invoking Cheque method in main net to send assets
// back to the withdraw issuer.
func (bp *Processor) processLock(lock *balanceEvent.Lock) {
	if !bp.alphabetState.IsAlphabet() {
		bp.log.Info(logs.BalanceNonAlphabetModeIgnoreBalanceLock)
		return
	}

	prm := frostfsContract.ChequePrm{}

	prm.SetID(lock.ID())
	prm.SetUser(lock.User())
	prm.SetAmount(bp.converter.ToFixed8(lock.Amount()))
	prm.SetLock(lock.LockAccount())
	prm.SetHash(lock.TxHash())

	err := bp.frostfsClient.Cheque(prm)
	if err != nil {
		bp.log.Error(logs.BalanceCantSendLockAssetTx, zap.Error(err))
	}
}
